==> moptipy-0.9.39-py3-none-any.whl: moptipy/evaluation/ioh_analyzer.py <==
"""Convert moptipy data to IOHanalyzer data."""

import argparse
import contextlib
from typing import Callable, Final

import numpy as np

from moptipy.evaluation.base import F_NAME_RAW, TIME_UNIT_FES, check_f_name
from moptipy.evaluation.progress import Progress
from moptipy.utils.console import logger
from moptipy.utils.help import argparser
from moptipy.utils.path import Path
from moptipy.utils.strings import num_to_str
from moptipy.utils.types import type_error


def __prefix(s: str) -> str:
    """
    Return `xxx` if `s` is of the form `xxx_i` and `i` is `int`.

    :param s: the function name
    :return: the dimension
    """
    idx = s.rfind("_")
    if idx > 0:
        with contextlib.suppress(ValueError):
            i = int(s[idx + 1:])
            if i > 0:
                return s[:idx].strip()
    return s


def __int_suffix(s: str) -> int:
    """
    Return `i` if `s` is of the form `xxx_i` and `i` is `int`.

    This function tries to check if the name

    :param s: the function name
    :return: the dimension
    """
    idx = s.rfind("_")
    if idx > 0:
        with contextlib.suppress(ValueError):
            i = int(s[idx + 1:])
            if i > 0:
                return i
    return 1


def moptipy_to_ioh_analyzer(
        results_dir: str,
        dest_dir: str,
        inst_name_to_func_id: Callable[[str], str] = __prefix,
        inst_name_to_dimension: Callable[[str], int] = __int_suffix,
        inst_name_to_inst_id: Callable[[str], int] = lambda x: 1,
        suite: str = "moptipy",
        f_name: str = F_NAME_RAW,
        f_standard: dict[str, int | float] | None = None) -> None:
    """
    Convert moptipy log data to IOHanalyzer log data.

    :param results_dir: the directory where we can find the results in moptipy
        format
    :param dest_dir: the directory where we would write the IOHanalyzer style
        data
    :param inst_name_to_func_id: convert the instance name to a function ID
    :param inst_name_to_dimension: convert an instance name to a function
        dimension
    :param inst_name_to_inst_id: convert the instance name an instance ID,
        which must be a positive integer number
    :param suite: the suite name
    :param f_name: the objective name
    :param f_standard: a dictionary mapping instances to standard values
    """
    source: Final[Path] = Path.directory(results_dir)
    dest: Final[Path] = Path.path(dest_dir)
    dest.ensure_dir_exists()
    logger(f"converting the moptipy log files in '{source}' to "
           f"IOHprofiler data in '{dest}'. First we load the data.")

    if (f_standard is not None) and (not isinstance(f_standard, dict)):
        raise type_error(f_standard, "f_standard", dict)
    if not isinstance(suite, str):
        raise type_error(suite, "suite", str)
    if (len(suite) <= 0) or (" " in suite):
        raise ValueError(f"invalid suite name '{suite}'")
    if not callable(inst_name_to_func_id):
        raise type_error(
            inst_name_to_func_id, "inst_name_to_func_id", call=True)
    if not callable(inst_name_to_dimension):
        raise type_error(
            inst_name_to_dimension, "inst_name_to_dimension", call=True)
    if not callable(inst_name_to_inst_id):
        raise type_error(
            inst_name_to_inst_id, "inst_name_to_inst_id", call=True)

    # the data
    data: Final[dict[str, dict[str, dict[int, list[
        tuple[int, np.ndarray, np.ndarray]]]]]] = {}

    # this consumer collects all the data in a structured fashion
    def __consume(progress: Progress) -> None:
        nonlocal data  # noqa
        nonlocal inst_name_to_func_id
        nonlocal inst_name_to_dimension
        nonlocal inst_name_to_inst_id

        _algo: dict[str, dict[int, list[tuple[int, np.ndarray, np.ndarray]]]]
        if progress.algorithm in data:
            _algo = data[progress.algorithm]
        else:
            data[progress.algorithm] = _algo = {}
        _func_id: Final[str] = inst_name_to_func_id(progress.instance)
        if not isinstance(_func_id, str):
            raise type_error(_func_id, "function id", str)
        if (len(_func_id) <= 0) or ("_" in _func_id):
            raise ValueError(f"invalid function id '{_func_id}'.")
        _func: dict[int, list[tuple[int, np.ndarray, np.ndarray]]]
        if _func_id in _algo:
            _func = _algo[_func_id]
        else:
            _algo[_func_id] = _func = {}
        _dim: Final[int] = inst_name_to_dimension(progress.instance)
        if not isinstance(_dim, int):
            raise type_error(_dim, "dim", int)
        if _dim <= 0:
            raise ValueError(f"invalid dimension: {_dim}.")
        _iid: Final[int] = inst_name_to_inst_id(progress.instance)
        if not isinstance(_iid, int):
            raise type_error(_iid, "instance id", int)
        if _iid <= 0:
            raise ValueError(f"invalid instance id: {_iid}.")
        _res: Final[tuple[int, np.ndarray, np.ndarray]] = \
            (_iid, progress.time, progress.f)
        if _dim in _func:
            _func[_dim].append(_res)
        else:
            _func[_dim] = [_res]

    Progress.from_logs(source,
                       consumer=__consume,
                       time_unit=TIME_UNIT_FES,
                       f_name=check_f_name(f_name),
                       f_standard=f_standard,
                       only_improvements=True)

    if len(data) <= 0:
        raise ValueError("did not find any data!")
    logger(f"finished loading data from {len(data)} algorithms, "
           "now writing output.")

    for algo_name in sorted(data.keys()):
        algo = data[algo_name]
        algo_dir: Path = dest.resolve_inside(algo_name)
        algo_dir.ensure_dir_exists()
        logger(f"writing output for {len(algo)} functions of "
               f"algorithm '{algo_name}'.")
        for func_id in sorted(algo.keys()):
            func_dir: Path = algo_dir.resolve_inside(f"data_f{func_id}")
            func_dir.ensure_dir_exists()
            func = algo[func_id]
            logger(f"writing output for algorithm '{algo_name}' and "
                   f"function '{func_id}', got {len(func)} dimensions.")

            func_name = f"IOHprofiler_f{func_id}"
            with algo_dir.resolve_inside(
                    f"{func_name}.info").open_for_write() as info:
                for dimi in sorted(func.keys()):
                    dim_path = func_dir.resolve_inside(
                        f"{func_name}_DIM{dimi}.dat")
                    info.write(f"suite = '{suite}', funcId = '{func_id}', "
                               f"DIM = {dimi}, algId = '{algo_name}'\n")
                    info.write("%\n")
                    info.write(dim_path[len(algo_dir) + 1:])
                    with dim_path.open_for_write() as dat:
                        for per_dim in sorted(
                                func[dimi], key=lambda x:
                                (x[0], x[2][-1], x[1][-1])):
                            info.write(f", {per_dim[0]}:")
                            fes = per_dim[1]
                            f = per_dim[2]
                            info.write(num_to_str(fes[-1]))
                            info.write("|")
                            info.write(num_to_str(f[-1]))
                            dat.write(
                                '"function evaluation" "best-so-far f(x)"\n')
                            for i, ff in enumerate(f):
                                dat.write(f"{num_to_str(fes[i])} "
                                          f"{num_to_str(ff)}\n")
                        dat.write("\n")
                info.write("\n")
    del data
    logger("finished converting moptipy data to IOHprofiler data.")


# Run conversion if executed as script
if __name__ == "__main__":
    parser: Final[argparse.ArgumentParser] = argparser(
        __file__,
        "Convert experimental results from the moptipy to the "
        "IOHanalyzer format.",
        "The experiment execution API of moptipy creates an output "
        "folder structure with clearly specified log files that can be"
        " evaluated with our experimental data analysis API. The "
        "IOHprofiler tool chain offers another format (specified in "
        "https://iohprofiler.github.io/IOHanalyzer/data/). With this "
        "tool here, you can convert from the moptipy to the "
        "IOHprofiler format.")
    parser.add_argument(
        "source", help="the directory with moptipy log files", type=Path.path,
        nargs="?", default="./results")
    parser.add_argument(
        "dest", help="the directory to write the IOHanalyzer data to",
        type=Path.path, nargs="?", default="./IOHanalyzer")
    args: Final[argparse.Namespace] = parser.parse_args()
    moptipy_to_ioh_analyzer(args.source, args.dest)

==> moptipy-0.9.46-py3-none-any.whl: moptipy/evaluation/ioh_analyzer.py <==
"""Convert moptipy data to IOHanalyzer data."""

import argparse
import contextlib
from typing import Callable, Final

import numpy as np

from moptipy.evaluation.base import F_NAME_RAW, TIME_UNIT_FES, check_f_name
from moptipy.evaluation.progress import Progress
from moptipy.utils.console import logger
from moptipy.utils.help import argparser
from moptipy.utils.path import Path
from moptipy.utils.strings import num_to_str
from moptipy.utils.types import type_error


def __prefix(s: str) -> str:
    """
    Return `xxx` if `s` is of the form `xxx_i` and `i` is `int`.

    :param s: the function name
    :return: the dimension
    """
    idx = s.rfind("_")
    if idx > 0:
        with contextlib.suppress(ValueError):
            i = int(s[idx + 1:])
            if i > 0:
                return s[:idx].strip()
    return s


def __int_suffix(s: str) -> int:
    """
    Return `i` if `s` is of the form `xxx_i` and `i` is `int`.

    This function tries to check if the name

    :param s: the function name
    :return: the dimension
    """
    idx = s.rfind("_")
    if idx > 0:
        with contextlib.suppress(ValueError):
            i = int(s[idx + 1:])
            if i > 0:
                return i
    return 1


def moptipy_to_ioh_analyzer(
        results_dir: str,
        dest_dir: str,
        inst_name_to_func_id: Callable[[str], str] = __prefix,
        inst_name_to_dimension: Callable[[str], int] = __int_suffix,
        inst_name_to_inst_id: Callable[[str], int] = lambda x: 1,
        suite: str = "moptipy",
        f_name: str = F_NAME_RAW,
        f_standard: dict[str, int | float] | None = None) -> None:
    """
    Convert moptipy log data to IOHanalyzer log data.

    :param results_dir: the directory where we can find the results in moptipy
        format
    :param dest_dir: the directory where we would write the IOHanalyzer style
        data
    :param inst_name_to_func_id: convert the instance name to a function ID
    :param inst_name_to_dimension: convert an instance name to a function
        dimension
    :param inst_name_to_inst_id: convert the instance name an instance ID,
        which must be a positive integer number
    :param suite: the suite name
    :param f_name: the objective name
    :param f_standard: a dictionary mapping instances to standard values
    """
    source: Final[Path] = Path.directory(results_dir)
    dest: Final[Path] = Path.path(dest_dir)
    dest.ensure_dir_exists()
    logger(f"converting the moptipy log files in {source!r} to "
           f"IOHprofiler data in {dest!r}. First we load the data.")

    if (f_standard is not None) and (not isinstance(f_standard, dict)):
        raise type_error(f_standard, "f_standard", dict)
    if not isinstance(suite, str):
        raise type_error(suite, "suite", str)
    if (len(suite) <= 0) or (" " in suite):
        raise ValueError(f"invalid suite name {suite!r}")
    if not callable(inst_name_to_func_id):
        raise type_error(
            inst_name_to_func_id, "inst_name_to_func_id", call=True)
    if not callable(inst_name_to_dimension):
        raise type_error(
            inst_name_to_dimension, "inst_name_to_dimension", call=True)
    if not callable(inst_name_to_inst_id):
        raise type_error(
            inst_name_to_inst_id, "inst_name_to_inst_id", call=True)

    # the data
    data: Final[dict[str, dict[str, dict[int, list[
        tuple[int, np.ndarray, np.ndarray]]]]]] = {}

    # this consumer collects all the data in a structured fashion
    def __consume(progress: Progress) -> None:
        nonlocal data  # noqa
        nonlocal inst_name_to_func_id
        nonlocal inst_name_to_dimension
        nonlocal inst_name_to_inst_id

        _algo: dict[str, dict[int, list[tuple[int, np.ndarray, np.ndarray]]]]
        if progress.algorithm in data:
            _algo = data[progress.algorithm]
        else:
            data[progress.algorithm] = _algo = {}
        _func_id: Final[str] = inst_name_to_func_id(progress.instance)
        if not isinstance(_func_id, str):
            raise type_error(_func_id, "function id", str)
        if (len(_func_id) <= 0) or ("_" in _func_id):
            raise ValueError(f"invalid function id {_func_id!r}.")
        _func: dict[int, list[tuple[int, np.ndarray, np.ndarray]]]
        if _func_id in _algo:
            _func = _algo[_func_id]
        else:
            _algo[_func_id] = _func = {}
        _dim: Final[int] = inst_name_to_dimension(progress.instance)
        if not isinstance(_dim, int):
            raise type_error(_dim, "dim", int)
        if _dim <= 0:
            raise ValueError(f"invalid dimension: {_dim}.")
        _iid: Final[int] = inst_name_to_inst_id(progress.instance)
        if not isinstance(_iid, int):
            raise type_error(_iid, "instance id", int)
        if _iid <= 0:
            raise ValueError(f"invalid instance id: {_iid}.")
        _res: Final[tuple[int, np.ndarray, np.ndarray]] = \
            (_iid, progress.time, progress.f)
        if _dim in _func:
            _func[_dim].append(_res)
        else:
            _func[_dim] = [_res]

    Progress.from_logs(source,
                       consumer=__consume,
                       time_unit=TIME_UNIT_FES,
                       f_name=check_f_name(f_name),
                       f_standard=f_standard,
                       only_improvements=True)

    if len(data) <= 0:
        raise ValueError("did not find any data!")
    logger(f"finished loading data from {len(data)} algorithms, "
           "now writing output.")

    for algo_name in sorted(data.keys()):
        algo = data[algo_name]
        algo_dir: Path = dest.resolve_inside(algo_name)
        algo_dir.ensure_dir_exists()
        logger(f"writing output for {len(algo)} functions of "
               f"algorithm {algo_name!r}.")
        for func_id in sorted(algo.keys()):
            func_dir: Path = algo_dir.resolve_inside(f"data_f{func_id}")
            func_dir.ensure_dir_exists()
            func = algo[func_id]
            logger(f"writing output for algorithm {algo_name!r} and "
                   f"function {func_id!r}, got {len(func)} dimensions.")

            func_name = f"IOHprofiler_f{func_id}"
            with algo_dir.resolve_inside(
                    f"{func_name}.info").open_for_write() as info:
                for dimi in sorted(func.keys()):
                    dim_path = func_dir.resolve_inside(
                        f"{func_name}_DIM{dimi}.dat")
                    info.write(f"suite = {suite!r}, funcId = {func_id!r}, "
                               f"DIM = {dimi}, algId = {algo_name!r}\n")
                    info.write("%\n")
                    info.write(dim_path[len(algo_dir) + 1:])
                    with dim_path.open_for_write() as dat:
                        for per_dim in sorted(
                                func[dimi], key=lambda x:
                                (x[0], x[2][-1], x[1][-1])):
                            info.write(f", {per_dim[0]}:")
                            fes = per_dim[1]
                            f = per_dim[2]
                            info.write(num_to_str(fes[-1]))
                            info.write("|")
                            info.write(num_to_str(f[-1]))
                            dat.write(
                                '"function evaluation" "best-so-far f(x)"\n')
                            for i, ff in enumerate(f):
                                dat.write(f"{num_to_str(fes[i])} "
                                          f"{num_to_str(ff)}\n")
                        dat.write("\n")
                info.write("\n")
    del data
    logger("finished converting moptipy data to IOHprofiler data.")


# Run conversion if executed as script
if __name__ == "__main__":
    parser: Final[argparse.ArgumentParser] = argparser(
        __file__,
        "Convert experimental results from the moptipy to the "
        "IOHanalyzer format.",
        "The experiment execution API of moptipy creates an output "
        "folder structure with clearly specified log files that can be"
        " evaluated with our experimental data analysis API. The "
        "IOHprofiler tool chain offers another format (specified in "
        "https://iohprofiler.github.io/IOHanalyzer/data/). With this "
        "tool here, you can convert from the moptipy to the "
        "IOHprofiler format.")
    parser.add_argument(
        "source", help="the directory with moptipy log files", type=Path.path,
        nargs="?", default="./results")
    parser.add_argument(
        "dest", help="the directory to write the IOHanalyzer data to",
        type=Path.path, nargs="?", default="./IOHanalyzer")
    args: Final[argparse.Namespace] = parser.parse_args()
    moptipy_to_ioh_analyzer(args.source, args.dest)
